/* From: http://stackoverflow.com/questions/4961095/help-understanding-abstract-classes-java-calendar-method-getinstance 
 * Abstract classes don't have to have any abstract methods. 
 * Having abstract methods is just how it's usually done.
 * 
 * Now Calendar.getInstance() works by actually creating an instance of a subclass. 
 * You're allowed to call it because it's a static method. 
 * The return value refers to an instance of the relevant subclass, 
 * but the return type is just Calendar, which is fine due to the normal rules of inheritance.
 * 
 * Here's an example of the same sort of approach, but without all the complexities 
 * of Calendar - the same principles apply.
 */
abstract class AbstractParent {
    public static AbstractParent getInstance() {
        // Normally you'd have code to work out which
        // concrete class to actually use
        return new ConcreteChild();
    }

    public abstract void sayHello();
}

class ConcreteChild extends AbstractParent {
    @Override public void sayHello() {
        System.out.println("Hello from ConcreteChild");
    }
}

public class AbstractGetInstanceTest {
    public static void main(String[] args) {
        AbstractParent parent = AbstractParent.getInstance();
        parent.sayHello();
        System.out.printf("\nThe variable 'parent' is of class: %s\n", parent.getClass().getName());
    }
}
